@@ -26,7 +26,7 @@ except ImportError:
 COLLECTION_RECENT = -2
 
 
-class _LibraryBackend:
+class _LibraryBackend(object):
 
     """The LibraryBackend handles the storing and retrieval of library
     data to and from disk.
@@ -34,7 +34,7 @@ class _LibraryBackend:
 
     #: Current version of the library database structure.
     # See method _upgrade_database() for changes between versions.
-    DB_VERSION = 5
+    DB_VERSION = 6
 
     def __init__(self):
 
@@ -153,11 +153,13 @@ class _LibraryBackend:
         """ Returns a pixbuf with a thumbnail of the cover of the book at <path>,
         or None, if no thumbnail could be generated. """
 
-        thumbnailer = thumbnail_tools.Thumbnailer(dst_dir=constants.LIBRARY_COVERS_PATH)
-        thumbnailer.set_store_on_disk(True)
-        # This is the maximum image size allowed by the library, so that thumbnails might be downscaled,
-        # but never need to be upscaled (and look ugly)
-        thumbnailer.set_size(constants.MAX_LIBRARY_COVER_SIZE, constants.MAX_LIBRARY_COVER_SIZE)
+        # Use the maximum image size allowed by the library, so that thumbnails
+        # might be downscaled, but never need to be upscaled (and look ugly).
+        thumbnailer = thumbnail_tools.Thumbnailer(dst_dir=constants.LIBRARY_COVERS_PATH,
+                                                  store_on_disk=True,
+                                                  archive_support=True,
+                                                  size=(constants.MAX_LIBRARY_COVER_SIZE,
+                                                        constants.MAX_LIBRARY_COVER_SIZE))
         thumb = thumbnailer.thumbnail(path)
 
         if thumb is None: log.warning( _('! Could not get cover for book "%s"'), path )
@@ -329,7 +331,7 @@ class _LibraryBackend:
 
                 book = backend_types._Book(book_id, name, path, pages,
                         format, size, datetime.datetime.now().isoformat())
-                self.book_added(book, collection)
+                self.book_added(book)
 
             cursor.close()
 
@@ -342,13 +344,23 @@ class _LibraryBackend:
             return False
 
     @callback.Callback
-    def book_added(self, book, collection):
+    def book_added(self, book):
         """ Event that triggers when a new book is successfully added to the
         library.
         @param book: L{_Book} instance of the newly added book.
         """
         pass
 
+    @callback.Callback
+    def book_added_to_collection(self, book, collection_id):
+        """ Event that triggers when a book is added to the
+        specified collection.
+        @param book: L{_Book} instance of the added book.
+        @param collection_id: ID of the collection.
+        """
+        pass
+
+
     def add_collection(self, name):
         """Add a new collection with <name> to the library. Return True
         if the collection was successfully added.
@@ -375,6 +387,8 @@ class _LibraryBackend:
         try:
             self._con.execute('''insert into Contain
                 (collection, book) values (?, ?)''', (collection, book))
+            self.book_added_to_collection(self.get_book_by_id(book),
+                collection)
         except dbapi2.DatabaseError: # E.g. book already in collection.
             pass
         except dbapi2.Error:
@@ -586,14 +600,30 @@ class _LibraryBackend:
                 lastread = last_read_page.LastReadPage(self)
                 lastread.migrate_database_to_library(COLLECTION_RECENT)
 
+            if 5 in upgrades:
+                # Changed all 'string' columns into 'text' columns
+                self._con.execute('''alter table book rename to book_old''')
+                self._create_table_book()
+                self._con.execute('''insert into book
+                    (id, name, path, pages, format, size, added)
+                    select id, name, path, pages, format, size, added from book_old''')
+                self._con.execute('''drop table book_old''')
+
+                self._con.execute('''alter table collection rename to collection_old''')
+                self._create_table_collection()
+                self._con.execute('''insert into collection
+                    (id, name, supercollection)
+                    select id, name, supercollection from collection_old''')
+                self._con.execute('''drop table collection_old''')
+
             self._con.execute('''update info set value = ? where key = 'version' ''',
                               (str(_LibraryBackend.DB_VERSION),))
 
     def _create_table_book(self):
         self._con.execute('''create table if not exists book (
             id integer primary key,
-            name string,
-            path string unique,
+            name text,
+            path text unique,
             pages integer,
             format integer,
             size integer,
@@ -602,7 +632,7 @@ class _LibraryBackend:
     def _create_table_collection(self):
         self._con.execute('''create table if not exists collection (
             id integer primary key,
-            name string unique,
+            name text unique,
             supercollection integer)''')
 
     def _create_table_contain(self):
