@@ -76,12 +76,13 @@ prefs = {
     'sharpness': 1.0,
     'auto contrast': False,
     'max pages to cache': 7,
+    'window x': 0,
+    'window y': 0,
     'window height': 600,
-    'window width': 500,
+    'window width': 640,
     'pageselector height': -1,
     'pageselector width': -1,
     'library cover size': 125,
-    'auto add books into collections': True,
     'last library collection': None,
     'lib window height': 600,
     'lib window width': 500,
@@ -91,8 +92,9 @@ prefs = {
     'statusbar fields': constants.STATUS_PAGE | constants.STATUS_RESOLUTION | \
                         constants.STATUS_PATH | constants.STATUS_FILENAME,
     'max threads': 3,
+    'max extract threads': 1,
     'wrap mouse scroll': False,
-    'scaling quality': 1,  # gtk.gdk.INTERP_TILES
+    'scaling quality': 2,  # gtk.gdk.INTERP_BILINEAR
     'escape quits': False,
     'fit to size mode': constants.ZOOM_MODE_HEIGHT,
     'fit to size px': 1800,
